%
%   [res]=FindCGHrelevantRegions (acgh,l,dok,fdrex,utili,param,plotyes,mrna) 
%
% INPUT:
% acgh  = struct array with the acgh data, the following fields are required:
%       .data     => matrix NxP with N samples and P DNA-probes
%       .gnames   => vector 1xP with the names of the DNA-clones
%       .ch       => vector 1xP with the chromosome in which the P probes are located   
%       .kbmidpos => vector 1xP with the midposition of the probes
%       .kbcumdis => vector 1xP with the cumulative position along the genome of the probes
%       .name => name of the data( to be used to save the results)
%        
% l     = vector Nx1 with the labels of the N samples (must be a two class problem)
% dok   = delta paramiter for the SAM analysis (if not known,use [] for
%         default delta=[0.1:0.01:1]);
% fdrex = desired false discovery rate    
%
% utili = struct array with useful strings for plots title and legend  
% param = struct array with parameter for the analysis 
% pyes  = binary variable: pyes==1 makes the matlab plots
% mrna  = optional mrna dataset with the same structure as acgh (if
%         provided the genes in the identified regions are provided
% 
% DESCRIPTION 
% 

% $$$ 	utili.cghname=cgh.name;
% $$$ 	utili.class1='class 1'; utili.class2='class 2';
% $$$ 	utili.title1='Median of the two classes, ';
% $$$ 	utili.title2='Relevant regions';
% $$$ utili.savedir='/';
% $$$ utili.savefilename='pippo';:

function [res]=Sirac (cgh,l,dok,fdrex,utili,param,plotyes,mrna)
	
	% INPUT CHECKING 
	
	% plotyes=0 => no plots are saved ---  plotyes=1 => the plots
        % are saved
	if nargin<7
		plotyes=0;
	end
	
	if nargin<6 	
		param.SAMiter = 1000;
		param.WINDOW = [250, 500,1000:1000:12000];  % length of half window
		param.th = 0.05 ;    % threshold for the Bonferroni correction 
		param.regioDetection='minW';
		param.numth=2  % minimum number of relevant windows 
		fprintf('WARNING: default param is used \n')
	else
		if ~exist('param','var') 
			error('the input variable "param" is missing')
		end
	end
	
	param.region='num'; % extra option in findRegions.m (an
                                 % earlier version of the software
                                 % implemented another way of detecting the
                                 % regions
	try
		tmp1 = param.SAMiter >0;
		tmp2 = param.WINDOW >0;
		tmp3 = param.th >= 0 & param.th <1 ;
		tmp4 = param.numth <=length(param.WINDOW);
		tmp5 = 10>1;
	catch
		error('the fields of the variable "param" are not valid');
	end
	if ~isfield(param,'regioDetection')
		param.regioDetection='minW';
	else tmp5=(~isempty(strmatch(param.regioDetection,'minW')) | ~isempty(strmatch(param.regioDetection,'mergeW') ))
	end
	
	if ~tmp1 | ~tmp2 | ~tmp3 | ~tmp4 | ~tmp5
		error('the fields of the variable "param" are not valid')
	end
	
	if nargin<5
		utili.class1='class A';
		utili.class1='class B';
		utili.title1= 'class A/class B: median of the two classes';
		utili.title1= 'class A/class B: relevant regions';
		utili.name='yourDatasat';
	else
		if ~exist('utili','var') 
			error('the input variable "utili" is missing')
		end
		if ( ~isfield(utili,'class1') |  ~isfield(utili,'class2')| ~isfield(utili,'title1') | ~isfield(utili,'title2') | ~isfield(utili,'name'))
			error('the variable "utili" does not contain the required fields')
		end
	end	
	
	if ~isfield(utili,'savedir')
		fprintf('WARNING: the results are not saved \n')
	end
	
	if nargin<4
		fdrex =0.05;
	end
	if isempty(fdrex)
		fdrex=0.05
	end
	
	if nargin<3 
		delta=[0.1:0.01:1];
		fprintf('WARNING:the delta of the SAM is being optimized  \n')
	end
	
	if nargin<2
		error('please, provide the label vector as input')
	end
	
	if length(unique(l))~=2
		error('There are more then two classes, Sirac requires a two class problem')
	end
	
	% check the fields in the input data
	if (~isfield(cgh,'data') | ~isfield(cgh,'gnames') | ~isfield(cgh,'ch') | ~isfield(cgh,'kbcumdis') | ~isfield(cgh,'kbmidpos') )  
		
		error('The input data does not contains the required fields')
	end
	if size(cgh.data,2)== length(l)
		cgh.data=cgh.data';
	end
	if size(cgh.data,1)~=length(l)
		error ('The number of samples does not match the label vector')
	end
	
	
	% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	%check if there are missing location or ch Y is included
	ccc=unique(cgh.ch);
	chbad=setdiff(unique(cgh.ch),[1:23]);
	if ~isempty (chbad)
		fprintf('WARNING:there are missing locations, or ch Y is included')
		fprintf('WARNING:these locations are excluded')
		indchbad=[];
		for i=1:length(chbad)
			iii=find(cgh.ch==chbad(i));
			indchbad=[indchbad,iii];
		end
		indchok=setdiff([1:length(cgh.ch)],indchbad);
		cgh=reduce_data('col',cgh,indchok);
	end	
	
	
	% in the res variable will be saved all results and input
        % necessary to recomputed
	res.param=param;
	res.dok=dok;
	res.fdrex=fdrex;
	res.utili=utili;
	
	% make a prtools dataset
	data=dataset(cgh.data,l,'prior',[0.5 0.5],'name',utili.name);
      

	% RUN SAM SEARCH

	if isempty(dok)
		if isfield(utili,'delta')
			[resSAM,d] = SAM_findth (data,fdrex,utili.delta,param.SAMiter,utili.savedir);
		else 
			[resSAM,d] = SAM_findth (data,fdrex);
		end
	else d=dok;
	end

        % if exist a value of d for the given fdr do the search	
	if ~isempty(d) 
	
		[j,delta,fdr,pfdr,q,s,ps,s0,n_called] = sam_my_out(data,d,param.SAMiter); 
		fprintf(1,'fdr=%5.4f  fdrex=%5.4f d=%5.2f n_called=%5.0f \n',[fdr,fdrex,d,n_called]);
	
		% ASK for going further
		YES= input('Do you want to go further with this value of d (YES => 1)? ');
		
		while (YES ~= 1)
			d=input('type a new value for d: ');
			[j,delta,fdr,pfdr,q,s,ps,s0,n_called] = sam_my_out(data,d,param.SAMiter); 
			fprintf(1,'fdr=%5.4f  fdrex=%5.4f d=%5.2f n_called=%5.0f \n',[fdr,fdrex,d,n_called]);
			YES= input('Do you want to go further with this value of	d (YES=1)? ')
		end
		
% $$$ 		while fdr>fdrex
% $$$ 			fprintf(1,'%5.4f\n',[fdr,fdrex]);
% $$$ 			warning('the fdr expected is different then the actual one')
% $$$ 			d=d+0.01;   %NBNBNB
% $$$ 			[j,delta,fdr,pfdr,q,s,ps,s0] = sam_my_out(data,d,param.SAMiter); 
% $$$ 		end
		res.SAMd=d;
		res.SAMfdr=fdr;
		
	% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	% if there are clones selected (and therefore regions), do the search	
	if ~isempty(j)	
			
	%----
	rel =reduce_data('col',cgh,j); %rel.data=[];
	rel.qSAM=q(j);
	rel.SAMj=j;
	rel.SAMfdr=fdr;
	rel.SAMq=q;
	rel.SAMs=s;
	
	% RUN THE CLONE SEARCH for all different window sizes

	relevantW={}; relevantWch=[]; regionW={}; 
	for i=1:length(param.WINDOW)
		param.window=param.WINDOW(i);
		try
		[region,relevantwindow,relevantCh]=window_on_relevant_clonesOK (cgh,rel,param);
		catch
			warning('FindCGHrelevantREgions at line 96')
		%	keyboard
		end
		
		relevantW{i}=relevantwindow;
		regionW{i}=region;
		try
			if size(relevantCh,1)~=1
				relevantCh=relevantCh';
			end
			relevantWch=[relevantWch,relevantCh];
		catch
			warning('aiutoooo')
		%	keyboard
		end
		
fprintf(1,'w');
	end
	fprintf(1,'\n')
	relevantWch=unique(relevantWch);

	res.relevantW=relevantW;
	res.relevantWch=relevantWch;
		
	% PLOT THE RESULTS
	utili.title1=[utili.title1,num2str(length(j)), ' relevant clones', ' fdr=',num2str(fdr),' delta=',num2str(delta)];
	plot_RelevantClone_2fig(cgh,l,rel,relevantW,utili,param.WINDOW/500,[])
	plotname=[utili.name,'_fdr_00',num2str(round(rel.SAMfdr*1000)),'_summaryGenome.eps'];
	
	try
	if isfield(utili,'savedir')
		saveplotname=[utili.savedir,plotname];
		eval('saveas(gcf,saveplotname,''epsc'')');	
	end
	catch
		fprintf(1,'error in line=246')
	%	keyboard
	end
        %-------------
	rel.NumWindows=[];
	rel.l=l;
	
	for i=1:length(relevantWch)
		ch=relevantWch(i);
		utili.title=[utili.name,' ch ',num2str(ch),' th: ',num2str(param.numth)];
try
		[indch,NumWindows]=plot_RelevantRegionXch_3fig(cgh,l,rel,relevantW,utili,param.WINDOW/500,ch,[],param);
		rel.NumWindows(indch)=NumWindows;
catch
	fprintf(1,'error in line=260')
	keyboard

		
end
		plotname=[utili.name,'_fdr_00',num2str(round(rel.SAMfdr*1000)),'_num',num2str(param.numth),'region_CH',num2str(ch),'.eps'];

		if isfield(utili,'savedir')
			saveplotname=[utili.savedir,plotname];
			eval('saveas(gcf,saveplotname,''epsc'')');	
		end
	end
	res.rel=rel;
	

	% NBNBNB FIND RELEVANT GENES IF EXPRESSION IS GIVEN

	if exist('mrna','var')
% old		[generw,generwXch]=FindGenesInRelevantRegions (mrna,res);
		[generw,generwXch]=FindGenesInRelevantRegions_final (mrna,res);
%		res.generw=generw;
%		res.generwXch=generwXch;
	end
	
	fprintf(1,'.');
	% SAVE THE MAT RESULTS
	if isfield(utili,'savedir')
		savename=[utili.savedir,utili.name,'_RES_delta0',num2str(round(res.SAMd*100)),'_fdr0',num2str(round(res.rel.SAMfdr*100))];
		eval(['save ',savename,' res']);
	end
end
	
else
fprintf('The SAM did not find any relevant DNA-probes for the selected FDR \n')
res.SAMres=resSAM;
end

