%PERMUTEFUNC - Repeatedly permute class labels in A and apply a function
%
%   OUT = PERMUTEFUNC (NPERMS,FUNC,A,PAR1,...)
%
% INPUT
%   NPERMS		Number of permutations
%		FUNC			Function to call on permuted datasets, should return a row vector
% 	A					Dataset to permute
%   PAR1,... 	Optional parameters to FUNC
%
% OUTPUT
%		OUT				Output (NPERMS x ? matrix) of calls to FUNC.
%
% DESCRIPTION
% Repeatedly permutes the class labels of A and calls FUNC(A,PAR1,...).
% The output of FUNC should be a row vector, which is placed in OUT.
% 
% SEE ALSO
% PVALS

% (c) Dick de Ridder, 2003
% Information & Communication Theory Group
% Faculty of Electrical Engineering, Mathematics and Computer Science
% Delft University of Technology, Mekelweg 4, 2628 CD Delft, The Netherlands

% Addition: Multiclass response, TA Knijnenburg, August 2004

function out = permutefunc (n_perms,func,a,varargin)

	% Check input.

	if (~isdataset(a))
		error ('input should be a dataset');
	end;

	if (nargin < 3)
		prwarning(2,'number of permutations not specified, assuming 1000');
		n_perms = 1000;
	end;

	[n_arrays,n_genes,n_classes] = getsize(a);

  if (n_classes < 2)
    error ('can only work on multi-class data');
  end;

	nlab = getnlab(a); nlablist = unique(nlab);

  for c = 1:n_classes
    ind{c} = find(nlab==nlablist(c));
    n(c)   = length(ind{c});
  end;

  if (any(n<1))
    error ('need at least 1 sample per class');
  end;

	out = zeros(n_perms,n_genes);

	for j = 1:n_perms	

	    if (mod(j,100) == 0), fprintf(1,'.'); end;
        
		% Create random permutation and distribute among classes.
		plab = nlab(randperm(n_arrays));

		% Add permuted labels to A and call FUNC.
		a = setnlab(a,plab);
	 	out(j,:) = feval(func,a,varargin{:});

	end;
    
    fprintf(1,'\n');
	
return
