%SAM - Significance analysis for microarrays
%
%   [J,DELTA,FDR,PFDR,Q,S,PS,S0] = SAM(A,DELTA,NPERMS)
%
% INPUT
%   A		   	Training dataset
%   DELTA  	Array of thresholds for s-statistic (default: 0:0.1:1)
%   NPERMS 	Number of permutations to use for estimating the (P)FDR and
%          	finding the optimal delta
% 
% OUTPUT
%		J			 	Indices of significant genes
%		DELTA		Optimal threshold-value
%		FDR			Estimated false discovery rate at DELTA
% 	PFDR		Estimated positive false discovery rate at DELTA
%		Q				Q-value for each gene
%		S				S-statistic for each gene
%		PS			Average S-statistic of permutation test for each gene
%		S0			Estimated fudge factor
%
% DESCRIPTION
% Applies significance analysis of microarrays (SAM) based on SSTATS, 
% optimising the threshold DELTA at which genes are called significant.
% Estimates the FDR and PFDR for the optimal solution found, and estimates
% a Q-value for each gene.
%
% REFERENCES
% [1] Tusher, V., Tibshirani, R. and Chu, G., "Significance analysis of
%     microarrays applied to the ionizing radiation response", PNAS 98,
%     pp. 5116-5124, 2001.
% [2] Storey, J.D. and Tibshirani, R., "Statistical significance for 
%     genome-wide studies", PNAS 100, pp 9440-9445, 2003.
% [3] Storey, J.D. and Tibshirani, R. "SAM thresholding and false discovery 
%     rates for detecting differential gene expression in DNA microarrays",
%     in: "The Analysis of Gene Expression Data: Methods and Software",
%			Parmigiani, G., Garrett, E.S., Irizarry, R.A. and Zeger, S.L. (editors),
%			Springer, New York, NY.
% [4] Storey, J.D., "A direct approach to false discovery rates", Journal of
%     the Royal Statistical Society, Series B, 64, pp. 479-498, 2002.
% [5] chu, G., Narasimhan, B., Tibshirani, R. and Tusher, V., "SAM - Users 
%     guide and technical document".
% [6] Ge, U., Dudoit, S. and Speed, T.P., "Resampling-based multiple testing
%     for microarray data analysis", Technical Report #633, Dept. of
%     Statistics, University of California, Berkeley, CA, USA, 2003.
%
% SEE ALSO
% PAMC, SSTATS

% REVISION HISTORY
% DR1, Dick, 6/1/2004 - Bugfix (serious).
% TK,  Theo, 27/8/2004 - Bugfix
%      Theo, 8/10/2004 - Update

% (c) Dick de Ridder, 2003
% Information & Communication Theory Group
% Faculty of Electrical Engineering, Mathematics and Computer Science
% Delft University of Technology, Mekelweg 4, 2628 CD Delft, The Netherlands

function [ind,delta_opt,fdr_opt,pfdr_opt,q,osstat,psstat_avg,s0,n_called] = 						sam_my_out (a,delta,n_perms,opt)
    if (nargin < 4)
		opt = 1;
	end;
	if (nargin < 3)
		n_perms = 1000;
	end;
	if (nargin < 2)
		delta = 0.1:0.1:1;
	end;

	[n_arrays,n_genes] = size(a);

	% Calculate SAM-statistics and sort them.

	[sstat,s0] = sstats(a); [osstat,ind] = sort(sstat);

	% Do the same for N_PERMS permutations of the labels.

	prwarning (10,'Permuting...\n');
	psstat = permutefunc(n_perms,@sstats,a,s0);
	for b = 1:n_perms
		psstat(b,:) = sort(psstat(b,:));
	end;
	psstat_avg = mean(psstat); 
    
	% For all threshold values in DELTA, find the number of significant genes
	% (CALLED) and false positives (FP).

	[n_fp,n_called,pos,called] = sam_thresholds(osstat,psstat,delta);
	pi0 = sam_pi0(psstat,osstat); n_fp = n_fp .* pi0;

	% Estimate the (positive) false discovery rate, (P)FDR. Make sure
	% that both X/0 and 0/0 give 1 (and no errors).

	fdr  = n_fp ./ n_called; 
	pfdr = (n_fp + realmin) ./ (n_called .* pos + realmin); 
	pfdr(find(~isfinite(pfdr))) = 1;

	% The Q-value of gene I is the minimum PFDR over the DELTAs for which 
	% gene I was called significant.

	for i = 1:n_genes
		q(i) = min([1 pfdr(find(called(:,i)))]);
	end;
    q(ind) = q;

	d_opt = 1;
	for d = 1:length(delta)
%		fprintf(1,'Delta = %.02f: #FP = %.02f, #CALLED = %.02f, FDR = %.02f, PFDR = %.02f\n', delta(d), n_fp(d), n_called(d), fdr(d), pfdr(d));
		if (pfdr(d) < pfdr(d_opt)), d_opt = d; end;
	end;

    if opt~=2
        ind = ind(find(called(d_opt,:)));
	    fdr_opt = fdr(d_opt); pfdr_opt = pfdr(d_opt); delta_opt = delta(d_opt);
    else
        ind = [ind' called'];
        fdr_opt = fdr; pfdr_opt = pfdr; delta_opt = delta;
    end

return
	
% SAM_THRESHOLDS
%
% Apply a number of thresholds DELTA to the ordered S-statistics given in 
% OSSTAT. Needs permuted statistics in PSSTAT.

function [n_fp,n_called,pos,called] = sam_thresholds (osstat,psstat,delta)

	psstat_avg = mean(psstat); 
	n_deltas   = length(delta);
	n_genes    = length(psstat_avg);
	n_perms    = size(psstat,1);
	statdiff   = osstat - psstat_avg;
    
	called = zeros(n_deltas,n_genes);
    cut_up = inf*ones(1,n_deltas);
    cut_low = -inf*ones(1,n_deltas);
    
    % For all thresholds DELTA...
    
	for d = 1:n_deltas

        % CALLED: indicates whether a gene was called significant for this DELTA. 
		% Find minimal statistic of significant positive genes: the upper cut.
        
        i1 = find(statdiff > delta(d));  
		if (length(i1)>=2)
			cut_up(d) = min(osstat(i1(2:end)));
            called(d,i1(2:end)) = 1;
        end;

		% Find minimal statistic of significant negative genes: the lower cut.

		i2 = find(-statdiff > delta(d));
		if (length(i2)>=2)
			cut_low(d) = max(osstat(i2(1:end-1)));
		    called(d,i2(1:end-1)) = 1;
		end;
        
		% N_CALLED: the number of significant genes.

        n_called(d) = sum(called(d,:));
        
		% Find how many genes are called significant, on average, at the same 
		% cut-levels the permuted sets, as an estimate of the number of
		% false positives FP.

		n_significant = zeros(n_perms,1);
		for b = 1:n_perms
			n_significant(b) = sum((psstat(b,:) <= cut_low(d))+(psstat(b,:) >= cut_up(d)));
		end;
		n_fp(d) = median(n_significant);

		% Find fraction of times any significant genes were found in the
		% permutations (for pFDR).

		pos(d) = length(find(n_significant > 0))/n_perms;

	end;
	
return

% SAM_PI0
%
% Estimate pi0, the proportion of null statistics. The rejection region
% is chosen such that half of the null statistics fall in it.

function pi0 = sam_pi0(psstat,osstat)

	p = size(psstat,2); psstat = psstat(:);

	q25 = prctile(psstat,25); q75 = prctile(psstat,75);

	pi0 = length(intersect(find(osstat>q25),find(osstat<q75))) ./ (0.5*p);
	
	pi0 = min(pi0,1);

return
