%SSTATS - Compute SAM-statistics on two-class data
%
%   S = SSTATS(A,S0)
%
% INPUT
%   A   A multiple-class dataset
%   S0  Fixed offset for denominator
%
% OUTPUT
%   S   SAM-statistics of the features in A
%
% DESCRIPTIOn
% S-statistics are the statistics used in SAM. Basically, they're two-tailed
% pooled-variance T-statistics, but with an estimated "fudge factor" S0
% in the denominator.
%
% SEE ALSO
% PLTSTATS

% (c) Dick de Ridder, 2003
% Information & Communication Theory Group
% Faculty of Electrical Engineering, Mathematics and Computer Science
% Delft University of Technology, Mekelweg 4, 2628 CD Delft, The Netherlands

% Addition: Multiclass response, TA Knijnenburg, August 2004
% Addition: modification to work without PRtools, Carmen Lai November 2007 

function [sval,s0,r,s] = sstats (a,s0)

	% Check input.

	if (nargin < 2)
		prwarning(4,'s0 not supplied, optimising');
		s0 = []; 
	end;

	if (~isdataset(a))
		error ('input should be a dataset');
	end

	[n_arrays,n_genes,n_classes] = getsize(a);
	
	if (n_classes < 2)
		error ('can only work on multi-class data');
	end;

	nlab = getnlab(a); nlablist = unique(nlab);
	
	for c = 1:n_classes
		ind{c} = find(nlab==nlablist(c));
		n(c)   = length(ind{c});
	end;
	
	if (any(n<1))
		error ('need at least one sample per class');
	end;

if n_classes==2;	
    
    % Calculate two-tailed SAM-statistics.

	a1 = +a(find(nlab==1),:); 
	a2 = +a(find(nlab==2),:); 
	m1 = mean(a1);    m2 = mean(a2);
	r = m2 - m1;
	s = sqrt ((((1/n(1))+(1/n(2))) / (n(1) + n(2) - 2)) * ...
              (sum((a1-ones(n(1),1)*m1).^2) + ...
               sum((a2-ones(n(2),1)*m2).^2)));

else
    
    %Calculate multiclass SAM_statistics
    
    m = zeros(n_classes,n_genes);m_mat = zeros(n_arrays,n_genes); 
    for k = 1:n_classes;
        m(k,:) = mean(+a(ind{k},:));
        m_mat(ind{k},:) = ones(n(k),1)*m(k,:);
    end;
    tm = mean(a);
    r = sqrt((sum(n)/prod(n))*(n*((m-(ones(n_classes,1)*tm)).^2)));    
    s = sqrt((1./sum(n-1))*sum(1./n)*(sum((a-m_mat).^2))) ;
    
end
	if (isempty(s0))
		s0 = find_s0(r,s);
	end;

	sval = r ./ (s+s0);

return

function s0 = find_s0(r,s)

	for alpha = 1:101, q(alpha) = prctile(s,alpha-1); end;
	
	alpha = 0:0.05:1;

	for a = 1:length(alpha)
		d = r./(s+q(floor(alpha(a)*100+1))+realmin);
		for j = 1:100
			ind1 = find(s>=q(j)); ind2 = find(s(ind1)<q(j+1)); ind = ind1(ind2);
			if (~isempty(ind))
				md   = median(d(ind)); 
				v(j) = median(abs(d(ind)-md)) / .64;
			else
				v(j) = 0;
			end;
		end;
		cv(a) = (std(v)+realmin)./(mean(v)+realmin);
	end;

	[dummy,ind] = min(cv); 
	s0 = prctile(s,floor(alpha(ind)*100));

return