%LDC Linear Bayes Normal Classifier (BayesNormal_1)
%
%   [W.R,S] = LDC(A,R,S)
%   W = A*LDC([],R,S);
% 
% INPUT
%   A    Dataset
%   R,S  Regularization parameters, 0 <= R,S <= 1
%        (optional; default: no regularization, i.e. R,S = 0)
% 
% OUTPUT
%   W    Linear Bayes Normal classifier
%   R    Regularization parameter used 
%   S    Regularization parameter used
%
% DESCRIPTION  
% Computation of the linear classifier between the classes of the dataset A
% by assuming normal densities with equal covariance matrices. The joint
% covariance matrix is the weighted (by a priori probabilities) average of
% the class covariance matrices. R and S (0 <= R,S <= 1) are regularization
% parameters used for finding the covariance matrix G by:
%
%      G = (1-R-S)*G + R*diag(diag(G)) + S*mean(diag(G))*eye(size(G,1))
%
% The use of soft labels is supported. The classification A*W is computed
% by NORMAL_MAP.
%
% If R and/or S are NaN the regularisation parameters are optimised by REGOPTC.
% The best result is usually already obtained by R = 0 and S = NaN.
%
% Note that A*(KLMS([],N)*NMC) performs a very similar operation by first
% pre-whitening the data in an N-dimensional space, followed by the
% nearest mean classifier. The regularization controlled by N is different
% from the above in LDC as it entirely removes small variance directions.
%
% To some extend LDC is also similar to FISHERC
%
% EXAMPLES
% See PREX_PLOTC.
%
% REFERENCES
% 1. R.O. Duda, P.E. Hart, and D.G. Stork, Pattern classification, 2nd edition, 
% John Wiley and Sons, New York, 2001.
% 2. A. Webb, Statistical Pattern Recognition, John Wiley & Sons, New York, 2002.
% 3. C. Liu and H. Wechsler, Robust Coding Schemes for Indexing and Retrieval
% from Large Face Databases, IEEE Transactions on Image Processing, vol. 9, 
% no. 1, 2000, 132-136.
%
%  SEE ALSO
%  MAPPINGS, DATASETS, REGOPTC, NMC, NMSC, LDC, UDC, QUADRC, NORMAL_MAP, FISHERC

% Copyright: R.P.W. Duin, r.p.w.duin@prtools.org
% Faculty EWI, Delft University of Technology
% P.O. Box 5031, 2600 GA Delft, The Netherlands

% $Id: ldc.m,v 1.7 2007/09/25 11:16:00 davidt Exp $

function [W,r,s] = ldc(a,r,s)

	prtrace(mfilename);

	if (nargin < 3)
		prwarning(4,'Regularisation parameter S not given, assuming 0.');
		s = 0; 
	end
	if (nargin < 2)
		prwarning(4,'Regularisation parameter R not given, assuming 0.');
		r = 0;
	end

	if (nargin < 1) | (isempty(a))  % No input arguments: 
		W = mapping(mfilename,{r,s}); % return an untrained mapping.
	
	elseif any(isnan([r,s]))    % optimize regularisation parameters
			defs = {0,0};
			parmin_max = [1e-8,9.9999e-1;1e-8,9.9999e-1];
			[W,r,s] = regoptc(a,mfilename,{r,s},defs,[2 1],parmin_max,testc([],'soft'));
	
	else % training

		islabtype(a,'crisp','soft');
		isvaldfile(a,2,2); % at least 2 object per class, 2 classes

		[m,k,c] = getsize(a);

		% Calculate mean vectors, priors and the covariance matrix G.
		%DXD: for large datasets (high dim, many classes) it is impossible
		%to keep all cov. matrices into memory. We have to loop over the
		%individual classes:
		%[U,G] = meancov(a);
		U = meancov(a);
		w.mean  = +U;
		w.prior = getprior(a);
		%G = reshape(sum(reshape(G,k*k,c)*w.prior',2),k,k);
		[tmpU,G] = meancov(seldat(a,1)); 
		G = w.prior(1)*G;
		for i=2:c
			[tmpU,tmpG] = meancov(seldat(a,i)); 
			G = G + w.prior(i)*tmpG;
		end
		clear tmpG;

		% Regularize 
		if (s > 0) | (r > 0)
			G = (1-r-s)*G + r * diag(diag(G)) + s*mean(diag(G))*eye(size(G,1));
		end
		w.cov = G;
		W = normal_map(w,getlab(U),k,c);
		W = setcost(W,a);
		
	end
	
	W = setname(W,'Bayes-Normal-1');
	
return
